
	list		p=16f877A	; list directive to define processor
	#include	<p16f877A.inc>	; processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _HS_OSC & _WRT_OFF & _LVP_OFF & _CPD_ON


;***** VARIABLE DEFINITIONS
w_temp		EQU	0x7D		; variable used for context saving 
status_temp	EQU	0x7E		; variable used for context saving
pclath_temp	EQU	0x7F		; variable used for context saving			

	cblock 0x20
target0						;target values for PWM counters
target1
target2
target3
target4
target5
target6
target7
target8
target9

present0					;present values of PWM counters
present1
present2
present3
present4
present5
present6
present7
present8
present9
random						;randomly generated number for PWM target
pwm_counter					;pwm reset and duration counter

	endc
	




;**********************************************************************
	ORG	0x000					;processor reset vector

	nop							;nop required for icd
	call	setup				;initialise registers
	goto	main				;go to beginning of program


	ORG		0x004				;interrupt vector location

	movwf	w_temp				;save off current W register contents
	movf	STATUS,w			;move status register into W register
	movwf	status_temp			;save off contents of STATUS register
	movf	PCLATH,w			;move pclath register into w register
	movwf	pclath_temp			;save off contents of PCLATH register
	
	banksel	PIR1
	btfss	PIR1,TMR1IF			;check if TMR1 roll over interrupt
	goto	ISR_restore			;no, so restore and exit

	call	step_to_target		;advance all the present values
	movlw	0xF6				;preset for timer1 MSB register (250Hz when xtal = 20MHz)
	movwf	TMR1H
	movlw	0x3C				;preset for timer1 LSB register
	movwf	TMR1L
	bcf		PIR1,TMR1IF			;clear the interrupt flag
	
ISR_restore
	movf	pclath_temp,w		;retrieve copy of PCLATH register
	movwf	PCLATH				;restore pre-isr PCLATH register contents
	movf	status_temp,w		;retrieve copy of STATUS register
	movwf	STATUS				;restore pre-isr STATUS register contents
	swapf	w_temp,f
	swapf	w_temp,w			;restore pre-isr W register contents
	retfie						;return from interrupt

setup
	movlw	0x55				;seed the random number generator
	movwf	random
	banksel	TRISA
	clrf	TRISA
	clrf	TRISB
	clrf	TRISC
	clrf	TRISD
	movlw	0x06				;for safety, make all PORTA pins digital outputs
	movwf	ADCON1
	banksel	OPTION_REG
	movlw	0x80				;no pull up on port B
	movwf	OPTION_REG
	banksel PORTA				
	clrf 	PORTA				;drive all unused PORTA bits low
	clrf	PORTC				;drive all unused PORTC bits low
	clrf	ADCON0				;disable the ADC module
	clrf	T1CON				;set up timer 1
	bsf		T1CON,T1CKPS1		;bits 5-4  Prescaler Rate Select bits
	bsf		T1CON,T1CKPS0
	bsf		T1CON,T1OSCEN		;bit 3 Timer1 Oscillator Enable Control: bit 1=on
	bsf		T1CON,T1SYNC		;bit 2 Timer1 External Clock Input Synchronization Control bit
	bcf		T1CON,TMR1CS		;bit 1 Timer1 Clock Source Select bit: 0=Internal clock (FOSC/4)
	bsf		T1CON,TMR1ON		;bit 0 enables timer
	movlw	0xF6				;preset for timer1 MSB register (250Hz when xtal = 20MHz)
	movwf	TMR1H
	movlw	0x3C				;preset for timer1 LSB register
	movwf	TMR1L
	banksel	PIE1				;enable timer1 interrupts
	bsf		PIE1,TMR1IE
	bsf		INTCON,PEIE			;enable peripheral interrupts
	bsf		INTCON,GIE			;enable global interrupts
	

	return
	
main
	call	rand_num_gen
	movwf	target0
	call	rand_num_gen
	movwf	target1
	call	rand_num_gen
	movwf	target2
	call	rand_num_gen
	movwf	target3
	call	rand_num_gen
	movwf	target4
	call	rand_num_gen
	movwf	target5
	call	rand_num_gen
	movwf	target6
	call	rand_num_gen
	movwf	target7
	call	rand_num_gen
	movwf	target8
	call	rand_num_gen
	movwf	target9

PWM_loop	
	incfsz	pwm_counter,f		;if zero, restart the PWM pulse period
	goto	pwm0
	banksel	PORTB
	movlw	0xFF
	movwf	PORTB				;turn all outputs ON
	movwf	PORTD
	goto	PWM_loop
pwm0
	movf	present0,w			;if pwm_counter matches the present value, turn RB bit off
	subwf	pwm_counter,w		;set Z flag if equal
	btfsc	STATUS,Z			;skip if not equal
	bcf		PORTB,0				;end of PWM period so turn bit off
	
	movf	present1,w
	subwf	pwm_counter,w
	btfsc	STATUS,Z
	bcf		PORTB,1
	
	movf	present2,w
	subwf	pwm_counter,w
	btfsc	STATUS,Z
	bcf		PORTB,2
	
	movf	present3,w
	subwf	pwm_counter,w
	btfsc	STATUS,Z
	bcf		PORTB,3
	
	movf	present4,w
	subwf	pwm_counter,w
	btfsc	STATUS,Z
	bcf		PORTB,4
	
	movf	present5,w
	subwf	pwm_counter,w
	btfsc	STATUS,Z
	bcf		PORTB,5
	
	movf	present6,w
	subwf	pwm_counter,w
	btfsc	STATUS,Z
	bcf		PORTB,6
	
	movf	present7,w
	subwf	pwm_counter,w
	btfsc	STATUS,Z
	bcf		PORTB,7

	movf	present8,w
	subwf	pwm_counter,w
	btfsc	STATUS,Z
	bcf		PORTD,0
	
	movf	present9,w
	subwf	pwm_counter,w
	btfsc	STATUS,Z
	bcf		PORTD,1
	
	goto	PWM_loop
	
	
rand_num_gen
	rlf		random,W
	rlf		random,W
	btfsc	random,4
	xorlw 	1
	btfsc 	random,5
	xorlw 	1
	btfsc 	random,3
	xorlw 	1
	movwf 	random
	return
	
step_to_target
	movf	target0,w			;read target value
	subwf	present0,w			;set flags on comparison
	btfss	STATUS,Z			;skip if target reached..
	goto	advance0			;..otherwise jump
	call	rand_num_gen
	movwf	target0				;target was reached so move it
advance0
	btfss	STATUS,C			;skip if present is higher than target
	goto	inc0
	decf	present0,f			;decrement toward target value
	goto	direction1
inc0
	incf	present0,f			;increment toward target
	
direction1
	movf	target1,w			;read target value
	subwf	present1,w			;set flags on comparison
	btfss	STATUS,Z			;skip if target reached..
	goto	advance1			;..otherwise jump
	call	rand_num_gen
	movwf	target1				;target was reached so move it
advance1
	btfss	STATUS,C			;skip if present is higher than target
	goto	inc1
	decf	present1,f			;decrement toward target value
	goto	direction2
inc1
	incf	present1,f			;increment toward target
	
direction2
	movf	target2,w			;read target value
	subwf	present2,w			;set flags on comparison
	btfss	STATUS,Z			;skip if target reached..
	goto	advance2			;..otherwise jump
	call	rand_num_gen
	movwf	target2				;target was reached so move it
advance2
	btfss	STATUS,C			;skip if present is higher than target
	goto	inc2
	decf	present2,f			;decrement toward target value
	goto	direction3
inc2
	incf	present2,f			;increment toward target
	
direction3
	movf	target3,w			;read target value
	subwf	present3,w			;set flags on comparison
	btfss	STATUS,Z			;skip if target reached..
	goto	advance3			;..otherwise jump
	call	rand_num_gen
	movwf	target3				;target was reached so move it
advance3
	btfss	STATUS,C			;skip if present is higher than target
	goto	inc3
	decf	present3,f			;decrement toward target value
	goto	direction4
inc3
	incf	present3,f			;increment toward target
	
direction4
	movf	target4,w			;read target value
	subwf	present4,w			;set flags on comparison
	btfss	STATUS,Z			;skip if target reached..
	goto	advance4			;..otherwise jump
	call	rand_num_gen
	movwf	target4				;target was reached so move it
advance4
	btfss	STATUS,C			;skip if present is higher than target
	goto	inc4
	decf	present4,f			;decrement toward target value
	goto	direction5
inc4
	incf	present4,f			;increment toward target
	
direction5
	movf	target5,w			;read target value
	subwf	present5,w			;set flags on comparison
	btfss	STATUS,Z			;skip if target reached..
	goto	advance5			;..otherwise jump
	call	rand_num_gen
	movwf	target5				;target was reached so move it
advance5
	btfss	STATUS,C			;skip if present is higher than target
	goto	inc5
	decf	present5,f			;decrement toward target value
	goto	direction6
inc5
	incf	present5,f			;increment toward target
	
direction6
	movf	target6,w			;read target value
	subwf	present6,w			;set flags on comparison
	btfss	STATUS,Z			;skip if target reached..
	goto	advance6			;..otherwise jump
	call	rand_num_gen
	movwf	target6				;target was reached so move it
advance6
	btfss	STATUS,C			;skip if present is higher than target
	goto	inc6
	decf	present6,f			;decrement toward target value
	goto	direction7
inc6
	incf	present6,f			;increment toward target
	
direction7
	movf	target7,w			;read target value
	subwf	present7,w			;set flags on comparison
	btfss	STATUS,Z			;skip if target reached..
	goto	advance7			;..otherwise jump
	call	rand_num_gen
	movwf	target7				;target was reached so move it
advance7
	btfss	STATUS,C			;skip if present is higher than target
	goto	inc7
	decf	present7,f			;decrement toward target value
	goto	direction8
inc7
	incf	present7,f			;increment toward target
	
direction8
	movf	target8,w			;read target value
	subwf	present8,w			;set flags on comparison
	btfss	STATUS,Z			;skip if target reached..
	goto	advance8			;..otherwise jump
	call	rand_num_gen
	movwf	target8				;target was reached so move it
advance8
	btfss	STATUS,C			;skip if present is higher than target
	goto	inc8
	decf	present8,f			;decrement toward target value
	goto	direction9
inc8
	incf	present8,f			;increment toward target

direction9
	movf	target9,w			;read target value
	subwf	present9,w			;set flags on comparison
	btfss	STATUS,Z			;skip if target reached..
	goto	advance9			;..otherwise jump
	call	rand_num_gen
	movwf	target9				;target was reached so move it
advance9
	btfss	STATUS,C			;skip if present is higher than target
	goto	inc9
	decf	present9,f			;decrement toward target value
	return
inc9
	incf	present9,f			;increment toward target
	return

	DT	"(C)Brian Kelly 2011"	
	
	END